@@ -1,10 +1,8 @@
 import re
-import shutil
 import threading
-from typing import Type
+from typing import Optional, Type
 from urllib.parse import urlparse, urljoin
 from os import path
-import sys
 import requests
 
 from uldlib.captcha import CaptchaSolver
@@ -45,12 +43,14 @@ class Page:
     numTorLinks: int
     alreadyDownloaded: int
 
-    def __init__(self, url, target_dir, parts, tor: TorRunner, conn_timeout=DEFAULT_CONN_TIMEOUT):
+    linkCache: Optional[LinkCache] = None
+
+    def __init__(self, url: str, temp_dir: str, parts: int, tor: TorRunner, conn_timeout=DEFAULT_CONN_TIMEOUT):
         """Check given url and if it looks ok GET the Uloz.to page and save it.
 
             Arguments:
                 url (str): URL of the page with file
-                target_dir (str): user defined output directory
+                temp_dir (str): directory where .ucache file will be created
                 parts (int): number of segments (parts)
                 tor: (TorRunner): tor runner instance
             Raises:
@@ -58,12 +58,20 @@ class Page:
         """
 
         self.url = strip_tracking_info(url)
-        self.target_dir = target_dir
+        self.temp_dir = temp_dir
         self.parts = parts
         self.tor = tor
         self.conn_timeout = conn_timeout
 
         parsed_url = urlparse(self.url)
+        if parsed_url.scheme == "":
+            # Common mistake when copied from addres bar without https://
+            self.url = "https://" + self.url
+            parsed_url = urlparse(self.url)
+
+        if not parsed_url.hostname:
+            raise RuntimeError(f"Invalid url {self.url}")
+
         self.pagename = parsed_url.hostname.capitalize()
         self.cli_initialized = False
         self.alreadyDownloaded = 0
@@ -201,9 +209,8 @@ class Page:
         """
 
         self.numTorLinks = 0
-        self.torRunning = False
         self.cacheEmpty = False
-        self.linkCache = LinkCache(path.join(self.target_dir, self.filename))
+        self.linkCache = LinkCache(path.join(self.temp_dir, self.filename))
 
         while not self.cacheEmpty:
             cached = self.linkCache.get()
@@ -217,26 +224,9 @@ class Page:
             if stop_event and stop_event.is_set():
                 break
 
-            if not self.torRunning:
-                print("Starting TOR...")
-                # tor started after cli initialized
-                try:
-                    self.tor.start(log_func=solver.log)
-                    self.torRunning = True
-                    proxies = {
-                        'http': 'socks5://127.0.0.1:' + str(self.tor.tor_ports[0]),
-                        'https': 'socks5://127.0.0.1:' + str(self.tor.tor_ports[0])
-                    }
+            self.tor.launch()  # ensure that TOR is running
 
-                except OSError as e:
-                    self._error_net_stat(
-                        f"Tor start failed: {e}, exiting.. try run program again..", solver.log)
-                    # remove tor data
-                    if path.exists(self.tor.ddir):
-                        shutil.rmtree(self.tor.ddir, ignore_errors=True)
-                    sys.exit(1)
-
-            # reload tor after 1. use or all except badCatcha case
+            # reload tor after 1. use or all except badCaptcha case
             reload = False
             if self.stats["all"] > 0 or reload:
                 self.tor.reload()
@@ -254,7 +244,7 @@ class Page:
                 if self.isDirectDownload:
                     solver.log(f"TOR get downlink (timeout {self.conn_timeout})")
                     resp = s.get(self.captchaURL,
-                                 headers=XML_HEADERS, proxies=proxies, timeout=self.conn_timeout)
+                                 headers=XML_HEADERS, proxies=self.tor.proxies, timeout=self.conn_timeout)
                 else:
                     solver.log(f"TOR get new CAPTCHA (timeout {self.conn_timeout})")
                     r = s.get(self.captchaURL, headers=XML_HEADERS)
@@ -285,7 +275,7 @@ class Page:
                     solver.log(f"CAPTCHA answer '{captcha_answer}' (timeout {self.conn_timeout})")
 
                     resp = s.post(self.captchaURL, data=captcha_data,
-                                  headers=XML_HEADERS, proxies=proxies, timeout=self.conn_timeout)
+                                  headers=XML_HEADERS, proxies=self.tor.proxies, timeout=self.conn_timeout)
 
                 # generate result or break
                 result = self._link_validation_stat(resp, solver.log)
@@ -298,6 +288,9 @@ class Page:
                     self.linkCache.add(dlink)
                     self.numTorLinks += 1
                     yield dlink
+                elif self.isDirectDownload:
+                    solver.log("Direct download does no seem to work, trying with captcha resolution instead...")
+                    self.isDirectDownload = False
 
             except requests.exceptions.ConnectionError:
                 self._error_net_stat(
